/******************************************************************************
*
*
* (c) S. Perzborn [15.04.06]
*
*
*
*******************************************************************************
*
* Disclaimer: This program is an example and should be used as such.
*             If you wish to use this program or parts of it in your application,
*             you must validate the code yourself. BECK IPC GmbH can not be held
*             responsible for the correct functioning or coding of this example.
*
* Module:     txweb.c
* Function:		This example gets a tektronix TDS220 display hartcopy via RS232
*							(EXT-Port) & Tektronix-LPT-Port to	the IPC@CHIP webserverpage
*							RS232 Parameter: 9600,8,N,1
*
*******************************************************************************

/******************************************************************************
* Includes
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <conio.h>
#include <clib.h>

/******************************************************************************
* Constants
******************************************************************************/
#define BOOL                  unsigned char
#define FALSE                 0
#define TRUE                  ( !FALSE )

#define PAGE_BUFFER_SIZE 6000
void SendString(int port, char *s);
void InitTectronix(int port);
void AbortHardcopy(int port);
void StartHardcopy(int port);
long ReadHTMLPage(char *filename);
int WriteData(char * filename);

static char pic_buffer[40000L]; // Bildspeicher
long pic_buffer_pointer=0;
static char pagebuffer[PAGE_BUFFER_SIZE];        // buffer for CGI page
static char buffer[PAGE_BUFFER_SIZE];            // buffer for CGI page
unsigned int  bufferlength=-1;
unsigned int receivecounter=0;

BOOL changed_flag=FALSE;
//int out_port=0;


enum  {START,WAITING,ABORT,RECEIVE,READY,STROBE} transmit_status;
#define IRQ 0  							// Strobe = IPC@CHIP Interrupt edge sensitive
#define DATA_IN_PORT 0x640  // Data from Tektronix LPT-Port
#define PCS 6


/******************************************************************************
* Interrupt service
******************************************************************************/
void huge Strobe_Interrupt(void)
{
	transmit_status=STROBE;
  pic_buffer[receivecounter++]=inp(DATA_IN_PORT);
}

/******************************************************************************
* write received BMP to file at RAM-Drive
******************************************************************************/
int WriteData(char *filename)
{
	FILE *file = NULL;                 // File pointer
	int status=0;
  file = fopen(filename, "wb+" );
	if( file == NULL )
  {
   	// Error
	   printf("Error: Could not open file %s!",filename);
     status=-1;
  }
	else
  {
 	  printf("please wait, writing data ...");
    fwrite(pic_buffer,1,receivecounter,file);
    printf("\n ready \n");
    fclose(file);
    status=0;
	}
	return(status);
}

/******************************************************************************
* read HTML page into a buffer
******************************************************************************/
long ReadHTMLPage(char *filename)
{
  FILE *file = NULL;
	// Read HTML Page
  bufferlength=0;
	file = fopen(filename,"rb" );
	if( file == NULL )
  {
   	// Error
	   sprintf( buffer, "Error: Could not open file %s!",filename);
  }
	else
  {
   	// Read file into buffer
		bufferlength = fread(pagebuffer,1,PAGE_BUFFER_SIZE-1,file);
		pagebuffer[bufferlength+1]=0;
	  fclose(file);
  	printf("HTML-File gelesen\n");
	}
	return(bufferlength);
}

/******************************************************************************
* send string via RS232 port
******************************************************************************/
void SendString(int port, char *s)
{
	fossil_flush_output(port);
	fossil_writeblock(port,(unsigned char *)s,strlen(s));
  fossil_putbyte_wait (port,13);
  //api_sleep(100);
	fossil_flush_output(port);
}

/******************************************************************************
* init Tektronix interface
******************************************************************************/
void InitTektronix(int port)
{
	SendString(port,"RS232:BAUD 9600\n");
	SendString(port,"RS232:HARDFLAGGING OFF\n");
	SendString(port,"RS232:SOFTFLAGGING ON\n");
	SendString(port,"RS232:TRANSMIT:TERMINATOR LF\n");
	SendString(port,"RS232:PARITY NONE\n");
	SendString(port,"HARDCOPY:FORMAT BMP\n");
	//SendString(port,"HARDCOPY:FORMAT DESK\n");
	//SendString(port,"HARDCOPY:LAYOUT LANDSCAPE\n");
	SendString(port,"HARDCOPY:LAYOUT PORTRAIT\n");
	//SendString(port,"HARDCOPY:PORT RS232\n");
	SendString(port,"HARDCOPY:PORT CEN\n");
}


/******************************************************************************
* abort Hardcopy (not used)
******************************************************************************/
void AbortHardcopy(int port)
{
	SendString(port,"HARDCOPY ABORT\n");
}

/******************************************************************************
* start Hardcopy
******************************************************************************/
void StartHardcopy(int port)
{
	pic_buffer_pointer=0;
	SendString(port,"*WAI\n");
	SendString(port,"HARDCOPY START\n");
}

/******************************************************************************
* cgiFunction()
******************************************************************************/
void huge _pascal cgiFunction( rpCgiPtr CgiRequest )
{

#define MAX_FORMNAME_LENGTH 200
#define MAX_FULLPATH_LENGTH 200

  FormItem formItem;                          // Name and valuie of a form item
  static char namePtr[MAX_FORMNAME_LENGTH];   // Buffer for form item's name
  static char valuePtr[MAX_FULLPATH_LENGTH];  // Buffer for form item's value
  unsigned int i;
  char *pStr;


	// Set pointers
  formItem.NamePtr = namePtr;
	formItem.ValuePtr = valuePtr;

  // Evaluate arguments
	changed_flag=FALSE;
  if (transmit_status==READY)
  {
	 	while( strlen( CgiRequest->fArgumentBufferPtr ) > 0 )
  	{
    	CGI_GetFormItem( CgiRequest->fArgumentBufferPtr, &formItem );
    	if( strcmp( formItem.NamePtr, "START" ) == 0 )
   	  	transmit_status=START;
    	if( CGI_GetNextItem( CgiRequest->fArgumentBufferPtr,
     		                   &CgiRequest->fArgumentBufferPtr ) != 0 )
 	   	break;
  	}
  }

	for (i=0;i<bufferlength;i++)
    buffer[i] = pagebuffer[i];
	pStr = strstr(buffer, "status_");
  if (transmit_status == READY)
  {
    if(pStr > buffer)
 	   	memcpy(pStr,"ready__",7);
  }
  else
  {
    if(pStr > buffer)
 	   	memcpy(pStr,"load___",7);
  }

	pStr = strstr(buffer, "messung.bmp");
  if (transmit_status == READY)
  {
	  if(pStr > buffer)
     	memcpy(pStr,"messung.bmp",11);
  }
  else
  {
 	  if(pStr > buffer)
	   	memcpy(pStr,"loading.jpg",11);
  }
  // Give page to the webserver
	CgiRequest->fHttpResponse = CgiHttpOk;
 	CgiRequest->fDataType = CGIDataTypeHtml;
 	CgiRequest->fResponseBufferPtr = buffer;
  CgiRequest->fResponseBufferLength = strlen( buffer );
}


/******************************************************************************
* main()
******************************************************************************/
int main(void)
{
  int blinkstatus=0;
  char pagename[]="start.htm";
  // Settings for serial communication
  unsigned int	port = 0;
	unsigned int	flow = 0; // 0=No, 1=RTS/CTS 2=XON/XOFF",
  unsigned int	divisor = 0;
  unsigned long baud = 9600;
	unsigned long maxbaud = 0L;  // Maximum baudrate possible on this CPU
  int fortschritt=1;
  CGI_Entry cgiEntryGet;         // Holds information on the CGI function
  // Init Interrupt
	pfe_enable_int(IRQ);
  pfe_set_edge_level_intr_mode (IRQ,1);

  pfe_enable_bus (0xFF, 1);
	pfe_enable_pcs (PCS);
	//	pfe_enable_pio (0,4); // enable PIO 0 Out High     ACK#
  // ACK# --> STROBE# ==> TDS220 wertet leider ACK nicht aus
  // Zeit steigende Flanke STROBE# (INT0) bis Port lesen = ca. 30us (SC13)
 	pfe_enable_pio (1,5); // enable PIO 1 Out Low      BUSY
	pfe_enable_pio (3,4); // enable PIO 3 Out High

  // Get maximum baudrate
  maxbaud = hal_get_frequencies( GET_FRQ_MAX_BAUD );
	port=0;
	flow=1; // 0=No, 1=RTS/CTS 2=XON/XOFF"

	 // Calculate clock divider
  divisor = (unsigned int)((2*maxbaud) / baud);
	divisor++;
  divisor >>= 1;
	// init fossil
  if( fossil_init( port ) != 0x1954 )
	{
    printf("\r\n Serial init failed\r\n");
   	return(1);
	}
  // init baudrate
	fossil_set_extctrl(port,divisor,0,8,1);
  if (flow)
	{
  	printf("\r\nUsing %s flow control\r\n", (flow==1) ? "RTS/CTS" : "XON/XOFF");
    if (flow == 1) // RTS/CTS
     	fossil_set_flowcontrol( port, FOSSIL_FLOWCTRL_RTSCTS );
	  if (flow == 2) // XON/XOFF
  	  fossil_set_flowcontrol( port, FOSSIL_FLOWCTRL_XONXOFF_SEND_RECV );
  }
	fossil_purge_output( port );    // purge buffers
  fossil_purge_input( port );			// purge buffers
	ReadHTMLPage(pagename);
   // Install CGI GET function
  cgiEntryGet.PathPtr = "start";        // Name of the page
  cgiEntryGet.method = CgiHttpGet;       // HTTP method
  cgiEntryGet.CgiFuncPtr = cgiFunction;  // Function called on browser request
  if( CGI_Install( &cgiEntryGet ) != 0 )
  {
    printf( "\r\nInstalling CGI function %s failed"
            "\r\n", cgiEntryGet.PathPtr );
    return (1);
  }
	CGI_SetMainPage ("start");
  api_sleep(100);
  InitTektronix(port);
  transmit_status=READY;
  BIOS_Set_Focus(FOCUS_APPLICATION);
  hal_write_pio(3,0);
  printf("TWI - Tektronix Web Interface\n");
  printf("v1.0 S.Perzborn [15.04.06]\n");
  // Main Loop
  while (!kbhit())
  {
	  api_sleep(50);
    switch (transmit_status)
    {
	    case START:
	      BIOS_Power_SaveEx(1); // full clock speed
 	      printf("fast mode\n");
	      printf("\nStarte Hardcopy\n");
        hal_install_isr (IRQ,1,Strobe_Interrupt);
  	  	StartHardcopy(port);
				transmit_status=WAITING;
	      printf("Empfange Daten, bitte warten ...\n");
  	    receivecounter=0;
  	 	  api_sleep(250);
	      blinkstatus=0;
    	  hal_write_pio(3,blinkstatus);
    		break;

     	case WAITING:
				break;

			case STROBE:
	      //working
  	  	transmit_status=RECEIVE;
        if (blinkstatus==1)
 	        blinkstatus=0;
   	    else
     	    blinkstatus=1;
       	hal_write_pio(3,blinkstatus);
				break;

    	case RECEIVE:
        hal_install_isr(IRQ, 0, NULL);    // deactivate interrupt
      	printf("Strobe Timeout \n");
    	  printf("empfangene Bytes: %u\n",receivecounter);
        WriteData("c:\\messung.bmp");
	      printf("Hardcopy fertig\n");
        blinkstatus=1;
    	  hal_write_pio(3,blinkstatus);
        transmit_status=READY;
 	      BIOS_Power_SaveEx(2); // 1/2 clock speed
 	      printf("slow mode\n");
      	break;
	    default:
    		break;
    }
  }
  blinkstatus=1;
  hal_write_pio(3,blinkstatus);
	printf("\nExit program\n");
	BIOS_Set_Focus(FOCUS_SHELL);	//set IO_focus to command shell (default)
  hal_install_isr(IRQ, 0, NULL);    // deactivate interrupt
  fossil_deinit(port);    // deinit fossil
  printf("\r\ndone");
  return(0);
}
// End of file

